@@ -3,6 +3,8 @@
 
 # shellcheck disable=SC1091
 . /usr/lib/eole/ihm.sh
+#shellcheck disable=SC1091,SC1090
+. "/etc/eole/samba4-vars.conf"
 
 #
 # affecte une valeur à la variable TEMP_PASSWORD
@@ -305,22 +307,6 @@ function mask2cdr()
 }
 
 
-#
-# calcul nb bit => mask réseau
-# ex: 9 ==> 255.255.252.0
-#
-function cdr2mask()
-{
-    # Number of args to shift, 255..255, first non-255 byte, zeroes
-    set -- $(( 5 - ($1 / 8) )) 255 255 255 255 $(( (255 << (8 - ($1 % 8))) & 255 )) 0 0 0
-    if [ "$1" -gt 1 ] ;
-    then
-        shift "$1"
-    else
-        shift
-    fi
-    echo "${1-0}.${2-0}.${3-0}.${4-0}"
-}
 
 #
 # AIM: Initialise the AD databases
@@ -606,7 +592,7 @@ function samba_init_additional()
         # In reality, Samba could in fact join and pretend it ran the correct functional level, but this has security consequences and is not generally considered safe.
         # The advice is to downgrade the forest (and domain) functional level on the Windows DC to 2008 R2
         # (and turn off all the associated features in 2012) before joining Samba.
-        echo "Le DC ${AD_DC_SYSVOL_REF} est déclaré comme 'windows' : pas de workaround à éxecuter."
+        echo "Le DC ${AD_DC_SYSVOL_REF} est déclaré comme 'windows' : pas de workaround à exécuter."
     else
         # https://wiki.samba.org/index.php/Verifying_and_Creating_a_DC_DNS_Record
         if [ "${AD_DC_SYSVOL_TYPE}" = "samba" ]
@@ -615,14 +601,14 @@ function samba_init_additional()
             then
                 echo "Le DC ${AD_DC_SYSVOL_REF} est déclaré comme 'samba'. "
                 echo "Il faut vérifier les entrées DNS suivant 'https://wiki.samba.org/index.php/Verifying_and_Creating_a_DC_DNS_Record' "
-                echo "Executer sur le DC distant : "
+                echo "Exécuter sur le DC distant : "
                 echo "   samba-tool dns add \$(hostname) ${AD_REALM} ${AD_HOST_NAME} ${AD_HOST_NAME}.${AD_REALM} A ${AD_HOST_IP} "
                 echo "   ldbsearch -H /usr/local/samba/private/sam.ldb '(invocationId=*)' --cross-ncs objectguid"
                 echo "Identifier le 'objectGuid' à utiliser"
                 echo "   samba-tool dns add \$(hostname) _msdcs.${AD_REALM} <objectGuid> CNAME ${AD_HOST_NAME}.${AD_REALM} "
                 echo "   tdbbackup -s .bak /var/lib/samba/private/idmap.ldb"
                 echo "Copier /var/lib/samba/private/idmap.ldb.bak sur le serveur additionel dans /var/lib/samba/private/idmap.ldb"
-                read -r -p "Taper 'entrée' une fois que les commandes ont été executées sur le DC de référence: "
+                read -r -p "Taper 'entrée' une fois que les commandes ont été exécutées sur le DC de référence: "
             else
                 ssh_on_dc "${AD_DC_SYSVOL_POST_JOIN_CMD}" "${AD_HOST_NAME}" "${AD_HOST_IP}"
                 ssh_on_dc tdbbackup -s .bak /var/lib/samba/private/idmap.ldb
@@ -638,7 +624,7 @@ function samba_init_additional()
 
     start_samba
 
-    echo "Execute Synchro Sysvol"
+    echo "Exécute Synchro Sysvol"
     JobSynchroSysvol
 
     # Samba 4.1 : replication of the SysVol share isn't implemented. If you make any changes on that share,
@@ -658,9 +644,11 @@ EOF
     samba_create_profiles_dir
     CreoleCat -t resolv.conf
 
-    update_system_account "gpo-${AD_HOST_NAME}"
-    # le group GpoAdmins a été crée à l'instance du 1er dc
-    samba-tool group addmembers "GpoAdmins" "gpo-${AD_HOST_NAME}"
+    if [[ -z "${AD_SERVER_MODE}" ]];then
+        update_system_account "gpo-${AD_HOST_NAME}"
+        # le group GpoAdmins a été crée à l'instance du 1er dc
+        samba-tool group addmembers "GpoAdmins" "gpo-${AD_HOST_NAME}"
+    fi
 
     touch "${AD_INSTANCE_LOCK_FILE}"
 }
@@ -714,8 +702,10 @@ function samba_init_member()
         fi
         echo
     done
-
-    if ! net ads join -U"${AD_DELEGATION}%${AD_DELEGATION_PASSWORD}" -I "${IP_SERVER}"
+    if [[ ! -z "${AD_SERVER_FULLNAME}" ]]; then
+        OPT=" -S $AD_SERVER_FULLNAME"
+    fi
+    if ! net ads join -U"${AD_DELEGATION}%${AD_DELEGATION_PASSWORD}" -I "${IP_SERVER}" ${OPT}
     then
         EchoRouge "Impossible de joindre le domaine."
         exit 1
@@ -739,11 +729,22 @@ function samba_init_member()
         # shellcheck disable=SC2128
         for IP_DNS in ${IP_DNS_LISTE}
         do
-            if dig "@${IP_DNS}" "${AD_HOST_NAME}.${AD_REALM}" | grep "${AD_HOST_IP}"
+            if tcpcheck 5 "${IP_DNS}:53" | grep -v failed
             then
-                echo "Entrée DNS présente sur ${IP_DNS}, Ok"
-                entreeDnsTrouvee=1
-                break
+                if dig "@${IP_DNS}" "${AD_HOST_NAME}.${AD_REALM}" | grep "${AD_HOST_IP}"
+                then
+                    echo "Entrée DNS présente sur ${IP_DNS}, Ok"
+                    entreeDnsTrouvee=1
+                    break
+                fi
+            else
+                # dans le cas de migrationX.sh, bind n'est pas démarré
+                if samba-tool dns query "${IP_SERVER}" "${AD_REALM}" "${AD_HOST_NAME}" A -U"${AD_DELEGATION}%${AD_DELEGATION_PASSWORD}"
+                then
+                    echo "Entrée DNS présente sur ${IP_DNS}, Ok"
+                    entreeDnsTrouvee=1
+                    break
+                fi
             fi
         done
         if [ "$entreeDnsTrouvee" -eq 1 ]
@@ -805,6 +806,7 @@ function samba_migrate_dns()
             echo "No DNS dynamic update on RODC"
         fi
         if [[ "${AD_DNS_BACKEND}" = "BIND9_DLZ" ]]; then
+            systemctl reset-failed named
             systemctl restart named
         fi
     fi
@@ -903,7 +905,10 @@ function samba_reconfigure()
             check_certificat_samba
             update_or_create_zones_dns
             update_or_create_dns
-            create_gpo_account
+            if [[ -z "${AD_SERVER_MODE}" ]]
+            then
+                create_gpo_account
+            fi
             update_system_accounts
             add_reference_domain
             echo "* Active Directory est initialisé, reload la configuration"
@@ -1084,7 +1089,6 @@ function get_keytabfile_for_account()
 #
 function update_system_account()
 {
-    local sevenDaysBefore
     local file_time
     local account
     local NEWPASSWORD
@@ -1103,8 +1107,6 @@ function update_system_account()
     then
         # les mots de passe sont mis à jour si plus de 7 jours
         # rappel: la régle par default est de 42 jours.
-        current_time=$( date +%s )
-        sevenDaysBefore=$(( current_time - ( 60 * 60 * 24 * 7 ) ))
         if [ -f "$passwordFile" ]
         then
             file_time=$(stat --format='%Y' "$passwordFile")
@@ -1112,30 +1114,28 @@ function update_system_account()
             # hum... le fichier n'existe plus ! je force la recréation
             file_time=0
         fi
-        if (( file_time < sevenDaysBefore ))
-        then
-            # j'utilise la fonction pwgen S=--secure N=--numerals C=--capitalize 1=one line, 42=42 chars
-            NEWPASSWORD="$(pwgen -scn1 42)"
+        # on reset le mot de passe à chaque reconfigure #35892
+        # j'utilise la fonction pwgen S=--secure N=--numerals C=--capitalize 1=one line, 42=42 chars
+        NEWPASSWORD="$(pwgen -scn1 42)"
 
-            # update
-            samba-tool user setpassword "${account}" --newpassword="${NEWPASSWORD}"
-            printf "%s" "${NEWPASSWORD}" >"${passwordFile}"
-            samba-tool domain exportkeytab "${keytabFile}" --principal="${account}@${AD_REALM^^}" -P
-        fi
+        # update
+        samba-tool user setpassword "${account}" --newpassword="${NEWPASSWORD}"
+        printf "%s" "${NEWPASSWORD}" >"${passwordFile}"
+        samba-tool domain exportkeytab "${keytabFile}" --principal="${account}@${AD_REALM^^}" -P
     else
         # j'utilise la fonction pwgen S=--secure N=--numerals C=--capitalize 1=one line, 42=42 chars
         NEWPASSWORD="$(pwgen -scn1 42)"
-        
-        #create 
+
+        #create
         if samba-tool user create "${account}" "${NEWPASSWORD}"
         then
             touch "${passwordFile}"
-            chmod 600 "${passwordFile}"         
+            chmod 600 "${passwordFile}"
             printf "%s" "${NEWPASSWORD}" >"${passwordFile}"
             samba-tool domain exportkeytab "${keytabFile}" --principal="${account}@${AD_REALM^^}" -P
         fi
     fi
-}   
+}
 
 
 #
@@ -1159,8 +1159,8 @@ function kinit_for_account()
 function update_system_accounts()
 {
     local account
-    
-    [ -d /var/lib/samba/private/managed_account ] || mkdir -p /var/lib/samba/private/managed_account    
+
+    [ -d /var/lib/samba/private/managed_account ] || mkdir -p /var/lib/samba/private/managed_account
     find /var/lib/samba/private/managed_account -name '*.pwd' | while read -r  pwdFile
     do
         account=$(basename "$pwdFile" .pwd)
@@ -1168,14 +1168,13 @@ function update_system_accounts()
     done
 }
 
-
 function create_gpo_account()
 {
-    if ! samba-tool group show GpoAdmins >/dev/null 2>&1   
+    if ! samba-tool group show GpoAdmins >/dev/null 2>&1
     then
         samba-tool group add GpoAdmins
         # TODO : à améliorer !
-        samba-tool group addmembers "Domain Admins" GpoAdmins   
+        samba-tool group addmembers "Domain Admins" GpoAdmins
     fi
 
     update_system_account "gpo-${AD_HOST_NAME}"
@@ -1287,7 +1286,7 @@ function workaround_policies()
                 rm -rf "/home/sysvol/${AD_REALM}/Policies/$ID/"
             fi
         fi
-    done 
+    done
 }
 
 function workaround_sysvol()
@@ -1312,32 +1311,32 @@ function workaround_sysvol()
             fi
         fi
     fi
-    
+
 }
 
 function samba_delete_gpo()
 {
     local GPONAME="$1"
-    
+
     echo "* Supprime GPO $GPONAME"
     if [ -z "$GPONAME" ]
     then
         echo "Usage: samba_delete_gpo <gpo_name>"
         return 1
     fi
-    
+
     if [ "${AD_SERVER_ROLE}" != "controleur de domaine" ]
     then
-        echo "Cette commande ne doit pas être éxecutée sur les serveurs membres."
+        echo "Cette commande ne doit pas être exécutée sur les serveurs membres."
         return 1
     fi
-    
+
     if [ "${AD_ADDITIONAL_DC}" != "non" ]
     then
-        echo "Cette commande ne doit pas être éxecutée sur les Dc Secondaires."
+        echo "Cette commande ne doit pas être exécutée sur les Dc Secondaires."
         return 2
     fi
-    
+
     GPO_ADMIN="gpo-${AD_HOST_NAME}"
     GPO_ADMIN_DN="${GPO_ADMIN}@${AD_REALM^^}"
     GPO_ADMIN_PWD_FILE=$(get_passwordfile_for_account "${GPO_ADMIN}")
@@ -1348,8 +1347,8 @@ function samba_delete_gpo()
     fi
     ADMIN_PWD="$(cat "${GPO_ADMIN_PWD_FILE}")"
     CREDENTIAL="${GPO_ADMIN_DN}%${ADMIN_PWD}"
-    
-    if gpo-tool helper show_by_name "$GPONAME" --attribut name -H "ldap://${AD_HOST_NAME}.${AD_REALM}" -U"$CREDENTIAL" 2>/dev/null 
+
+    if gpo-tool helper show_by_name "$GPONAME" --attribut name -H "ldap://${AD_HOST_NAME}.${AD_REALM}" -U"$CREDENTIAL" 2>/dev/null
     then
         gpo-tool helper delete_by_name "$GPONAME" -H "ldap://${AD_HOST_NAME}.${AD_REALM}" -U"$CREDENTIAL" 1>/dev/null
         CDU="$?"
@@ -1357,8 +1356,8 @@ function samba_delete_gpo()
         CDU="0"
     fi
     workaround_policies
-    
-    return "$CDU" 
+
+    return "$CDU"
 }
 
 function samba_import_gpo_clean_after()
@@ -1369,7 +1368,7 @@ function samba_import_gpo_clean_after()
         if [ -n "$GPONAME" ]
         then
             /bin/rm -rf "/var/tmp/$GPONAME"
-        fi 
+        fi
     fi
 }
 
@@ -1379,7 +1378,7 @@ function samba_import_gpo()
     local SOURCE="$2"
     local DN_TO_LINK="$3"
     local SOURCE_TMP=""
-    
+
     #echo "* Import GPO $GPONAME from export $SOURCE"
     if [ -z "$GPONAME" ]
     then
@@ -1389,16 +1388,16 @@ function samba_import_gpo()
 
     if [ "${AD_SERVER_ROLE}" != "controleur de domaine" ]
     then
-        echo "Cette commande ne doit pas être éxecutée sur les serveurs membres."
+        echo "Cette commande ne doit pas être exécutée sur les serveurs membres."
         return 1
     fi
-    
+
     if [ "${AD_ADDITIONAL_DC}" != "non" ]
     then
-        echo "Cette commande ne doit pas être éxecutée sur les Dc Secondaires."
+        echo "Cette commande ne doit pas être exécutée sur les Dc Secondaires."
         return 2
     fi
-    
+
     GPO_ADMIN="gpo-${AD_HOST_NAME}"
     GPO_ADMIN_DN="${GPO_ADMIN}@${AD_REALM^^}"
     GPO_ADMIN_PWD_FILE=$(get_passwordfile_for_account "${GPO_ADMIN}")
@@ -1409,7 +1408,7 @@ function samba_import_gpo()
     fi
     ADMIN_PWD="$(cat "${GPO_ADMIN_PWD_FILE}")"
     CREDENTIAL="${GPO_ADMIN_DN}%${ADMIN_PWD}"
-    
+
     if [ ! -f "$SOURCE" ]
     then
         if [ ! -d "$SOURCE" ]
@@ -1427,13 +1426,13 @@ function samba_import_gpo()
         then
             mkdir "$SOURCE_TMP"
         fi
-        if ! pushd "$SOURCE_TMP" >/dev/null 
+        if ! pushd "$SOURCE_TMP" >/dev/null
         then
             echo "Impossible de changer de répertoire $SOURCE_TMP"
             samba_import_gpo_clean_after
             return 6
         fi
-        if ! tar xf "$SOURCE"
+        if ! tar zxf "$SOURCE"
         then
             echo "Impossible d'extraire le fichier '$SOURCE', est-ce un tar.gz ?"
             samba_import_gpo_clean_after
@@ -1455,10 +1454,10 @@ function samba_import_gpo()
         samba_import_gpo_clean_after
         return 7
     fi
-    
+
     #echo "* Fix netlogon"
-    gpo-tool helper fix_netlogon_scripts_acl  -H "ldap://${AD_HOST_NAME}.${AD_REALM}" -U"$CREDENTIAL" 
-    
+    gpo-tool helper fix_netlogon_scripts_acl  -H "ldap://${AD_HOST_NAME}.${AD_REALM}" -U"$CREDENTIAL"
+
     if GPOID=$(gpo-tool helper show_by_name "$GPONAME" --attribut name -H "ldap://${AD_HOST_NAME}.${AD_REALM}" -U"$CREDENTIAL" 2>/dev/null)
     then
         if [ ! -f "/var/tmp/gpo-script/update_$GPONAME" ]
@@ -1469,16 +1468,18 @@ function samba_import_gpo()
         fi
         echo "* '$GPONAME' existe, rebuild demandé"
         /bin/rm "/var/tmp/gpo-script/update_$GPONAME"
-         
+
         if ! gpo-tool helper delete_by_name "$GPONAME" -H "ldap://${AD_HOST_NAME}.${AD_REALM}" -U"$CREDENTIAL" >/dev/null 2>&1
         then
             echo "* Delete '$GPONAME' Erreur"
             # j'ignore si elle a déjà été supprimée !
         fi
     fi
-    
+
     echo "* Import GPO"
-    if ! samba-tool gpo restore -H "ldap://${AD_HOST_NAME}.${AD_REALM}" -U"${CREDENTIAL}" "$GPONAME" "$SOURCE/policy" --restore-metadata >/tmp/gpo_import.err 2>&1
+    samba-tool domain exportkeytab --principal=Administrator /var/lib/samba/private/managed_account/Administrator.keytab > /dev/null 2>&1
+    kinit -k -t /var/lib/samba/private/managed_account/Administrator.keytab Administrator
+    if ! samba-tool gpo restore --tmpdir /tmp -H "ldap://${AD_HOST_NAME}.${AD_REALM}" -N --use-kerberos=required "$GPONAME" "$SOURCE/policy" --restore-metadata >/tmp/gpo_import.err 2>&1
     then
         cat /tmp/gpo_import.err
         echo "* '$GPONAME' restauration impossible !"
@@ -1490,9 +1491,9 @@ function samba_import_gpo()
     then
         echo "* '$GPONAME' impossible de créer la GPO !"
         samba_import_gpo_clean_after
-        return 9 
+        return 9
     fi
-    
+
     if [ -f "$SOURCE/ldif" ]
     then
         VERSION_AD=$(gpo-tool helper show_by_name "$GPONAME" --attribut versionNumber -H "ldap://${AD_HOST_NAME}.${AD_REALM}" -U"$CREDENTIAL" )
@@ -1506,24 +1507,24 @@ changetype: modify
 replace: versionNumber
 versionNumber: ${VERSION_EXPORT}
 EOF
-    
+
             if ! ldbmodify -v -H "/var/lib/samba/private/sam.ldb" -U"${CREDENTIAL}" /tmp/gpoUpdate.ldif >/tmp/ldbmodify.err 2>&1
             then
                 cat /tmp/ldbmodify.err
-            fi 
+            fi
         fi
     fi
-    
+
     if [ -f "$SOURCE/attr" ]
     then
         echo "* Restauration xattr"
         cd "/home/sysvol/${AD_REALM}/Policies/$GPOID" || return 1
-        setfattr --restore "$SOURCE/attr" . 
+        setfattr --restore "$SOURCE/attr" .
     fi
-    
+
     #echo "* Fix GPO"
-    gpo-tool helper fix_gpo_acl "$GPONAME" -H "ldap://${AD_HOST_NAME}.${AD_REALM}" -U"$CREDENTIAL" 
-    
+    gpo-tool helper fix_gpo_acl "$GPONAME" -H "ldap://${AD_HOST_NAME}.${AD_REALM}" -U"$CREDENTIAL"
+
     # link ?
     if [ -n "$DN_TO_LINK" ]
     then
@@ -1534,14 +1535,14 @@ EOF
                 echo "  '$GPONAME' impossible de lier la GPO à '$DN_TO_LINK'"
                 cat /tmp/gpo_setlink.err
                 samba_import_gpo_clean_after
-                return 10 
+                return 10
             fi
         fi
     fi
-    
+
     workaround_policies
     workaround_sysvol
-    
+
     samba_import_gpo_clean_after
     return 0
 }
@@ -1550,7 +1551,7 @@ function samba_export_gpo()
 {
     local GPO_NAME="$1"
     local EXPORT_TAR_GZ="$2"
-    
+
     if [ -z "$GPO_NAME" ]
     then
         echo "Usage: samba_export_gpo <gpo_name> [<export_gpos.tar.gz>]"
@@ -1559,7 +1560,7 @@ function samba_export_gpo()
     if [ -z "$EXPORT_TAR_GZ" ]
     then
         EXPORT_TAR_GZ="/usr/share/eole/gpo/${GPO_NAME}.tar.gz"
-    fi    
+    fi
     if [ -f "$EXPORT_TAR_GZ" ]
     then
         echo "Usage: samba_export_gpo <gpo_name> [<export_gpos.tar.gz>]"
@@ -1567,19 +1568,19 @@ function samba_export_gpo()
         return 1
     fi
     echo "* Export GPO $GPO_NAME to $EXPORT_TAR_GZ"
-    
+
     if [ "${AD_SERVER_ROLE}" != "controleur de domaine" ]
     then
-        echo "Cette command ne doit pas être éxecutée sur les serveurs membres"
+        echo "Cette command ne doit pas être exécutée sur les serveurs membres"
         return 0
     fi
-    
+
     if [ "${AD_ADDITIONAL_DC}" != "non" ]
     then
-        echo "Cette commande ne doit pas être éxecutée sur les Dc Secondaires."
+        echo "Cette commande ne doit pas être exécutée sur les Dc Secondaires."
         return 0
     fi
-    
+
     GPO_ADMIN="gpo-${AD_HOST_NAME}"
     GPO_ADMIN_DN="${GPO_ADMIN}@${AD_REALM^^}"
     GPO_ADMIN_PWD_FILE=$(get_passwordfile_for_account "${GPO_ADMIN}")
@@ -1590,14 +1591,14 @@ function samba_export_gpo()
     fi
     ADMIN_PWD="$(cat "${GPO_ADMIN_PWD_FILE}")"
     CREDENTIAL="${GPO_ADMIN_DN}%${ADMIN_PWD}"
-    
+
     GPO_ID=$(gpo-tool helper show_by_name "$GPO_NAME" --attribut name -H "ldap://${AD_HOST_NAME}.${AD_REALM}" -U"${CREDENTIAL}" )
     if [ -z "$GPO_ID" ]
     then
         echo "La GPO $GPO_NAME n'existe pas!"
         return 1
     fi
-    
+
     rm -rf "/var/tmp/$GPO_NAME"
     mkdir -p "/var/tmp/$GPO_NAME"
     samba-tool gpo backup "$GPO_ID" --tmpdir="/var/tmp/$GPO_NAME" -d 0
@@ -1610,7 +1611,7 @@ function samba_export_gpo()
     getfacl -R . >"/var/tmp/$GPO_NAME/acls"
     find . |while read -r F ; do file "$F"; done >"/var/tmp/$GPO_NAME/encodings"
     find . |while read -r F ; do SDDL=$(samba-tool ntacl get  "$F" --as-sddl); echo "$SDDL $F"; done >"/var/tmp/$GPO_NAME/sddl"
-    
+
     echo "${AD_DOMAIN}" >"/var/tmp/$GPO_NAME/domain"
     gpo-tool helper show_by_name "$GPO_NAME" --attribut name -H "ldap://${AD_HOST_NAME}.${AD_REALM}" -U"${CREDENTIAL}" >"/var/tmp/$GPO_NAME/version"
     samba-tool ntacl get "/home/sysvol/${AD_REALM}/Policies/$GPO_ID/" --as-sddl >"/var/tmp/$GPO_NAME/sddl.FS_GPO"
@@ -1619,11 +1620,82 @@ function samba_export_gpo()
     samba-tool dsacl get --objectdn="CN=USer,CN=$GPO_ID,CN=Policies,CN=System,${BASEDN}"  -U"${CREDENTIAL}" | tail -1 >"/var/tmp/$GPO_NAME/sddl.FS_GPO_User"
     samba-tool ntacl get "/home/sysvol/${AD_REALM}/Policies/$GPO_ID/Machine" --as-sddl >"/var/tmp/$GPO_NAME/sddl.FS_GPO_Machine"
     samba-tool dsacl get --objectdn="CN=Machine,CN=$GPO_ID,CN=Policies,CN=System,${BASEDN}" -U"${CREDENTIAL}" | tail -1 >"/var/tmp/$GPO_NAME/sddl.DS_GPO_Machine"
-    
+
     cd "/var/tmp/$GPO_NAME/" || return 1
-    tar --xattrs --acls -cvf "$EXPORT_TAR_GZ" .
+    tar --xattrs --acls -zcvf "$EXPORT_TAR_GZ" .
     ls -l "$EXPORT_TAR_GZ"
-    
+
     echo "* Export GPO $GPO_NAME from export $EXPORT_TAR_GZ"
     return 0
 }
+
+function samba_gpo_load_credentials()
+{
+    BASEDN="DC=${AD_REALM//./,DC=}"
+    GPO_ADMIN="gpo-${AD_HOST_NAME}"
+    GPO_ADMIN_DN="${GPO_ADMIN}@${AD_REALM^^}"
+    GPO_ADMIN_PWD_FILE=${CONTAINER_ROOTFS}$(get_passwordfile_for_account "${GPO_ADMIN}")
+    if [ ! -f "${GPO_ADMIN_PWD_FILE}" ]
+    then
+        echo "Warning:Le fichier ${GPO_ADMIN_PWD_FILE} est manquant"
+        return 1
+    fi
+    ADMIN_PWD="$(cat "${GPO_ADMIN_PWD_FILE}")"
+    CREDENTIAL="${GPO_ADMIN_DN}%${ADMIN_PWD}"
+    return 0
+
+}
+
+function samba_link_gpo()
+{
+    GPONAME="$1"
+    OU="$2"
+    if ! GPOID=$(gpo-tool helper show_by_name "$GPONAME" --attribut name -H "ldap://${AD_HOST_NAME}.${AD_REALM}" -U"$CREDENTIAL" )
+    then
+        #echo "* '$GPONAME' n'existe pas !"
+        return
+    fi
+
+    if [ "$OU" == "" ]
+    then
+        local NEW_DN="$BASEDN"
+    else
+        local NEW_DN="OU=${OU//\//,OU=},$BASEDN"
+    fi
+    if ! samba-tool ou list --full-dn |grep -q "${NEW_DN}"
+    then
+        echo "'$NEW_DN' n'existe pas !"
+        return
+    fi
+
+    if samba-tool gpo setlink "$NEW_DN" "$GPOID" -H "ldap://${AD_HOST_NAME}.${AD_REALM}" -U"${CREDENTIAL}" >/dev/null 2>/tmp/gpo_setlink.err
+    then
+        echo "  '$GPONAME' liée à '$NEW_DN' "
+    else
+        if grep -q 'already linked to this container' /tmp/gpo_setlink.err
+        then
+            echo "  '$GPONAME' liée à '$NEW_DN'"
+        else
+            echo "  '$GPONAME' impossible de lier la GPO à '$NEW_DN'"
+            cat /tmp/gpo_setlink.err
+        fi
+    fi
+    samba_import_gpo_clean_after
+}
+
+#
+# calcul nb bit => mask réseau
+# ex: 9 ==> 255.255.252.0
+#
+function cdr2mask()
+{
+    # Number of args to shift, 255..255, first non-255 byte, zeroes
+    set -- $(( 5 - ($1 / 8) )) 255 255 255 255 $(( (255 << (8 - ($1 % 8))) & 255 )) 0 0 0
+    if [ "$1" -gt 1 ] ;
+    then
+        shift "$1"
+    else
+        shift
+    fi
+    echo "${1-0}.${2-0}.${3-0}.${4-0}"
+}
